@@ -1,7 +1,6 @@
-""" Demonstration of Pmw error handling.
-"""
+title = 'Pmw error handling demonstration'
 
-# Import Pmw from the sibling directory.
+# Import Pmw from this directory tree.
 import sys
 sys.path[:0] = ['../../..']
 
@@ -35,7 +34,7 @@ class Demo:
 if __name__ == '__main__':
     root = Tkinter.Tk()
     Pmw.initialise(root, fontScheme = 'pmw1')
-    root.title('Pmw error handling demonstration')
+    root.title(title)
 
     exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
     exitButton.pack(side = 'bottom')
