@@ -1,7 +1,6 @@
-""" Demonstration of the Pmw interface to the blt busy command.
-"""
+title = 'Blt busy cursor demonstration'
 
-# Import Pmw from the sibling directory.
+# Import Pmw from this directory tree.
 import sys
 sys.path[:0] = ['../../..']
 
@@ -41,7 +40,7 @@ class Demo:
 if __name__ == '__main__':
     root = Tkinter.Tk()
     Pmw.initialise(root, fontScheme = 'pmw1')
-    root.title('Busy cursor demonstration')
+    root.title(title)
 
     exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
     exitButton.pack(side = 'bottom')
