@@ -1,7 +1,6 @@
-""" Demonstration of the Pmw TimeCounter megawidget.
-"""
+title = 'Pmw.TimeCounter demonstration'
 
-# Import Pmw from the sibling directory.
+# Import Pmw from this directory tree.
 import sys
 sys.path[:0] = ['../../..']
 
@@ -33,7 +32,7 @@ class Demo:
 if __name__ == '__main__':
     root = Tkinter.Tk()
     Pmw.initialise(root, fontScheme = 'pmw1')
-    root.title('Pmw TimeCounter demonstration')
+    root.title(title)
 
     exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
     exitButton.pack(side = 'bottom')
