@@ -1,7 +1,6 @@
-""" Demonstration of megawidget destruction.
-"""
+title = 'Demonstration of Pmw megawidget destruction'
 
-# Import Pmw from the sibling directory.
+# Import Pmw from this directory tree.
 import sys
 sys.path[:0] = ['../../..']
 
@@ -11,17 +10,17 @@ import Pmw
 class Demo:
     def __init__(self, parent):
 	# Create and pack an EntryField.
-	self.entry = Pmw.EntryField(parent,
+	self.entryfield = Pmw.EntryField(parent,
 	    command = self.execute,
+            value = 'Press <Return> to destroy me',
 	    entry_width = 30)
-	self.entry.pack(fill='x', expand=1, padx=10, pady=5)
-	self.entry.insert(0, 'Press <Return> to destroy me')
+	self.entryfield.pack(fill='x', expand=1, padx=10, pady=5)
 
-	self.entry.component('entry').focus_set()
+	self.entryfield.component('entry').focus_set()
 
     def execute(self):
 	print 'Return pressed, destroying EntryField.'
-	self.entry.destroy()
+	self.entryfield.destroy()
 
 ######################################################################
 
@@ -29,7 +28,7 @@ class Demo:
 if __name__ == '__main__':
     root = Tkinter.Tk()
     Pmw.initialise(root, fontScheme = 'pmw1')
-    root.title('Pmw megawidget destruction demonstration')
+    root.title(title)
 
     exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
     exitButton.pack(side = 'bottom')
