@@ -1,10 +1,8 @@
-""" Demonstration of the Pmw EXAMPLE megawidget.
-"""
+title = 'Pmw.EXAMPLE demonstration'
 
-if __name__ == '__main__':
-    # Import Pmw from the sibling directory.
-    import sys
-    sys.path[:0] = ['../../..']
+# Import Pmw from this directory tree.
+import sys
+sys.path[:0] = ['../../..']
 
 import Tkinter
 import Pmw
@@ -27,7 +25,7 @@ class Demo:
 if __name__ == '__main__':
     root = Tkinter.Tk()
     Pmw.initialise(root, fontScheme = 'pmw1')
-    root.title('Pmw EXAMPLE demonstration')
+    root.title(title)
 
     exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
     exitButton.pack(side = 'bottom')
