@@ -21,10 +21,15 @@ import traceback
 import types
 import Tkinter
 
+# Special values used in index() methods of several megawidgets.
+END = ['end']
+SELECT = ['select']
+DEFAULT = ['default']
+
 # Constant used to indicate that an option can only be set by a call
 # to the constructor.
-INITOPT = [42]
-_DEFAULT_OPTION_VALUE = [69]
+INITOPT = ['initopt']
+_DEFAULT_OPTION_VALUE = ['default_option_value']
 _useTkOptionDb = 0
 
 # Symbolic constants for the indexes into an optionInfo list.
@@ -59,7 +64,8 @@ _grabStack = []
     #                       the grab on this window and restore the grab
     #                       on the next window in the stack (if there is one).
     #   'globalMode' :      True if the grabWindow was grabbed with a
-    #                       global grab and false if the grab was local.
+    #                       global grab, false if the grab was local
+    #                       and 'nograb' if no grab was performed.
     #   'previousFocus' :   The focus as it was when pushgrab()
     #                       was called.  The corresponding call to
     #                       popgrab() will restore this focus.
@@ -221,6 +227,33 @@ def forwardmethods(fromClass, toClass, toPart, exclude = ()):
 
 #=============================================================================
 
+def setgeometryanddeiconify(window, geom):
+    # To avoid flashes on X and to position the window correctly on NT
+    # (caused by Tk bugs).
+
+    if os.name == 'nt':
+        # Require overrideredirect trick to stop window frame
+        # appearing momentarily.
+        redirect = window.overrideredirect()
+        if not redirect:
+            window.overrideredirect(1)
+        window.deiconify()
+        if geom is not None:
+            window.geometry(geom)
+        # Call update_idletasks to ensure NT moves the window to the
+        # correct position it is raised.
+        window.update_idletasks()
+        window.tkraise()
+        if not redirect:
+            window.overrideredirect(0)
+    else:
+        if geom is not None:
+            window.geometry(geom)
+        window.deiconify()
+        window.tkraise()
+
+#=============================================================================
+
 class MegaArchetype:
     # Megawidget abstract root class.
 
@@ -273,6 +306,12 @@ class MegaArchetype:
 	#
 	# self._constructorKeywords = {}
 
+        # List of dynamic component groups.  If a group is included in
+        # this list, then it not an error if a keyword argument for
+        # the group is given to the constructor or to configure(), but
+        # no components with this group have been created.
+        # self._dynamicGroups = ()
+
 	if hullClass is None:
 	    self._hull = None
 	else:
@@ -292,10 +331,10 @@ class MegaArchetype:
 		# constructor.  This assumes that defineoptions() is
 		# called before the __init__().
 		option_get = self.option_get
-		VALUE = _OPT_VALUE
-		DEFAULT = _OPT_DEFAULT
+		_VALUE = _OPT_VALUE
+		_DEFAULT = _OPT_DEFAULT
 		for name, info in self._optionInfo.items():
-		    value = info[VALUE]
+		    value = info[_VALUE]
 		    if value is _DEFAULT_OPTION_VALUE:
 			resourceClass = string.upper(name[0]) + name[1:]
 			value = option_get(name, resourceClass)
@@ -305,14 +344,24 @@ class MegaArchetype:
 				value = eval(value, {'__builtins__': {}})
 			    except:
 				pass
-			    info[VALUE] = value
+			    info[_VALUE] = value
 			else:
-			    info[VALUE] = info[DEFAULT]
+			    info[_VALUE] = info[_DEFAULT]
+
+    def destroy(self):
+        # Clean up optionInfo in case it contains circular references
+        # in the function field, such as self._settitle in class
+        # MegaToplevel.
+
+	self._optionInfo = {}
+        if self._hull is not None:
+            del _hullToMegaWidget[self._hull]
+            self._hull.destroy()
 
     #======================================================================
     # Methods used (mainly) during the construction of the megawidget.
 
-    def defineoptions(self, keywords, optionDefs):
+    def defineoptions(self, keywords, optionDefs, dynamicGroups = ()):
 	# Create options, providing the default value and the method
 	# to call when the value is changed.  If any option created by
 	# base classes has the same name as one in <optionDefs>, the
@@ -329,6 +378,9 @@ class MegaArchetype:
 	    self._constructorKeywords = tmp
 	    self._optionInfo = {}
 
+        if not hasattr(self, '_dynamicGroups'):
+            self._dynamicGroups = ()
+        self._dynamicGroups = self._dynamicGroups + tuple(dynamicGroups)
 	self.addoptions(optionDefs)
 
     def addoptions(self, optionDefs):
@@ -370,18 +422,19 @@ class MegaArchetype:
 		if not keywords_has_key(name):
 		    keywords[name] = [default, 0]
 
-    def createcomponent(self, name, aliases, group, widgetClass,
-            *widgetArgs, **kw):
+    def createcomponent(self, componentName, componentAliases,
+            componentGroup, widgetClass, *widgetArgs, **kw):
 	# Create a component (during construction or later).
 
-	if '_' in name:
-	    raise ValueError, 'Component name "%s" must not contain "_"' % name
+	if '_' in componentName:
+	    raise ValueError, \
+                    'Component name "%s" must not contain "_"' % componentName
 
 	if hasattr(self, '_constructorKeywords'):
 	    keywords = self._constructorKeywords
 	else:
 	    keywords = {}
-	for alias, component in aliases:
+	for alias, component in componentAliases:
 	    # Create aliases to the component and its sub-components.
 	    index = string.find(component, '_')
 	    if index < 0:
@@ -403,10 +456,10 @@ class MegaArchetype:
 		    keywords[newkey] = keywords[option]
 		    del keywords[option]
 
-	componentName = name + '_'
-	nameLen = len(componentName)
+	componentPrefix = componentName + '_'
+	nameLen = len(componentPrefix)
 	for option in keywords.keys():
-	    if len(option) > nameLen and option[:nameLen] == componentName:
+	    if len(option) > nameLen and option[:nameLen] == componentPrefix:
 		# The keyword argument refers to this component, so add
 		# this to the options to use when constructing the widget.
 		kw[option[nameLen:]] = keywords[option][0]
@@ -419,7 +472,7 @@ class MegaArchetype:
 		# since it may be required when creating another
 		# component.
 		index = string.find(option, '_')
-		if index >= 0 and group == option[:index]:
+		if index >= 0 and componentGroup == option[:index]:
 		    rest = option[(index + 1):]
 		    kw[rest] = keywords[option][0]
 		    keywords[option][1] = 1
@@ -436,8 +489,8 @@ class MegaArchetype:
             widgetArgs = widgetArgs[0]
 	widget = apply(widgetClass, widgetArgs, kw)
 	componentClass = widget.__class__.__name__
-	self.__componentInfo[name] = (widget, widget.configure,
-		componentClass, widget.cget, group)
+	self.__componentInfo[componentName] = (widget, widget.configure,
+		componentClass, widget.cget, componentGroup)
 
 	return widget
 
@@ -489,7 +542,12 @@ class MegaArchetype:
 	    for name in keywords.keys():
 		used = keywords[name][1]
 		if not used:
-		    unusedOptions.append(name)
+                    # This keyword argument has not been used.  If it
+                    # does not refer to a dynamic group, mark it as
+                    # unused.
+                    index = string.find(name, '_')
+                    if index < 0 or name[:index] not in self._dynamicGroups:
+                        unusedOptions.append(name)
 	    self._constructorKeywords = {}
 	    if len(unusedOptions) > 0:
 		if len(unusedOptions) == 1:
@@ -608,7 +666,8 @@ class MegaArchetype:
 			    if info[4] == component:
 			        componentConfigFuncs.append(info[1])
 
-			if len(componentConfigFuncs) == 0:
+                        if len(componentConfigFuncs) == 0 and \
+                                component not in self._dynamicGroups:
 			    raise KeyError, 'Unknown option "' + option + \
 				    '" for ' + self.__class__.__name__
 
@@ -635,6 +694,9 @@ class MegaArchetype:
 	    if func is not None:
 	      func()
 
+    def __setitem__(self, key, value):
+        apply(self.configure, (), {key: value})
+
     #======================================================================
     # Methods used to query the megawidget.
 
@@ -790,7 +852,7 @@ def popgrab(window):
 
     # If this window is not at the top of the grab stack, then it has
     # just been deleted by the window manager or deactivated by a
-    # time.  Call the deactivate method for the modal dialog above
+    # timer.  Call the deactivate method for the modal dialog above
     # this one on the stack. 
     if _grabStack[-1]['grabWindow'] != window:
         for index in range(len(_grabStack)):
@@ -803,8 +865,11 @@ def popgrab(window):
 
     topWidget = grabInfo['grabWindow']
     prevFocus = grabInfo['previousFocus']
+    globalMode = grabInfo['globalMode']
+
+    if globalMode != 'nograb':
+        topWidget.grab_release()
 
-    topWidget.grab_release()
     if len(_grabStack) > 0:
         _grabtop()
     if prevFocus != '':
@@ -822,6 +887,12 @@ def popgrab(window):
         else:
             Tkinter._default_root.focus_set()
 
+def grabstacktopwindow():
+    if len(_grabStack) == 0:
+        return None
+    else:
+        return _grabStack[-1]['grabWindow']
+
 def releasegrabs():
     # Release grab and clear the grab stack.
 
@@ -835,6 +906,9 @@ def _grabtop():
     topWidget = grabInfo['grabWindow']
     globalMode = grabInfo['globalMode']
 
+    if globalMode == 'nograb':
+        return
+
     while 1:
         try:
             if globalMode:
@@ -867,7 +941,13 @@ class MegaToplevel(MegaArchetype):
 
 	# Initialise instance.
 
-	self.protocol('WM_DELETE_WINDOW', self._userDeleteWindow)
+        # Set WM_DELETE_WINDOW protocol, deleting any old callback, so
+        # memory does not leak.
+        if hasattr(self._hull, '_Pmw_WM_DELETE_name'):
+            self._hull.tk.deletecommand(self._hull._Pmw_WM_DELETE_name)
+        self._hull._Pmw_WM_DELETE_name = \
+                self.register(self._userDeleteWindow, needcleanup = 0)
+	self.protocol('WM_DELETE_WINDOW', self._hull._Pmw_WM_DELETE_name)
 
 	# Initialise instance variables.
 
@@ -910,9 +990,13 @@ class MegaToplevel(MegaArchetype):
     def destroy(self):
 	# Allow this to be called more than once.
 	if _hullToMegaWidget.has_key(self._hull):
-	    del _hullToMegaWidget[self._hull]
 	    self.deactivate()
-	    self._hull.destroy()
+
+            # Remove circular references, so that object can get cleaned up.
+            del self._userDeleteFunc
+            del self._userModalDeleteFunc
+
+            MegaArchetype.destroy(self)
 
     def show(self):
 	if self.state() == 'normal':
@@ -922,33 +1006,39 @@ class MegaToplevel(MegaArchetype):
 		# Just let the window manager determine the window
 		# position for the first time.
 	        self._firstShowing = 0
+		geom = None
 	    else:
 		# Position the window at the same place it was last time.
-		self._sameposition()
-	    self.deiconify()
-            if os.name == "nt":
-                # deiconify does not raise. Bug in Tk?
-                self.tkraise()
+		geom = self._sameposition()
+            setgeometryanddeiconify(self, geom)
+        self.focus()
 
     def _centreonscreen(self):
 	# Centre the window on the screen.  (Actually halfway across
 	# and one third down.)
 
-	self.update_idletasks()
-
         parent = self.winfo_parent()
         if type(parent) == types.StringType:
             parent = self._hull._nametowidget(parent)
 
-	x = (self.winfo_screenwidth() - self.winfo_reqwidth()) / 2 \
-		- parent.winfo_vrootx()
-	y = (self.winfo_screenheight() - self.winfo_reqheight()) / 3 \
-		- parent.winfo_vrooty()
+        # Find size of window.
+	self.update_idletasks()
+        width = self.winfo_width()
+        height = self.winfo_height()
+        if width == 1 and height == 1:
+            # If the window has not yet been displayed, its size is
+            # reported as 1x1, so use requested size.
+            width = self.winfo_reqwidth()
+            height = self.winfo_reqheight()
+
+        # Place in centre of screen:
+	x = (self.winfo_screenwidth() - width) / 2 - parent.winfo_vrootx()
+	y = (self.winfo_screenheight() - height) / 3 - parent.winfo_vrooty()
 	if x < 0:
 	    x = 0
 	if y < 0:
 	    y = 0
-	self.geometry('+%d+%d' % (x, y))
+        return '+%d+%d' % (x, y)
 
     def _sameposition(self):
 	# Position the window at the same place it was last time.
@@ -956,9 +1046,11 @@ class MegaToplevel(MegaArchetype):
 	geometry = self.geometry()
 	index = string.find(geometry, '+')
 	if index >= 0:
-	    self.geometry(geometry[index:])
+	    return geometry[index:]
+        else:
+	    return None
 
-    def activate(self, globalMode=0, geometry = 'centerscreenfirst'):
+    def activate(self, globalMode = 0, geometry = 'centerscreenfirst'):
 	if self._active:
 	    raise ValueError, 'Window is already active'
 	if self.state() == 'normal':
@@ -966,6 +1058,35 @@ class MegaToplevel(MegaArchetype):
 
 	self._active = 1
 
+	showbusycursor()
+
+	if self._wait is None:
+	    self._wait = Tkinter.IntVar()
+	self._wait.set(0)
+
+	if geometry == 'centerscreenalways':
+	    geom = self._centreonscreen()
+	elif geometry == 'centerscreenfirst':
+	    if self._firstShowing:
+		# Centre the window the first time it is displayed.
+		geom = self._centreonscreen()
+	    else:
+		# Position the window at the same place it was last time.
+		geom = self._sameposition()
+	elif geometry[:5] == 'first':
+	    if self._firstShowing:
+                geom = geometry[5:]
+	    else:
+		# Position the window at the same place it was last time.
+		geom = self._sameposition()
+        else:
+            geom = geometry
+
+	self._firstShowing = 0
+
+        setgeometryanddeiconify(self, geom)
+
+        # Do this last, otherwise get flashing on NT:
         master = self['master']
         if master is not None:
             if master == 'parent':
@@ -977,33 +1098,6 @@ class MegaToplevel(MegaArchetype):
                 master = parent.winfo_toplevel()
             self.transient(master)
 
-	showbusycursor()
-
-	if self._wait is None:
-	    self._wait = Tkinter.IntVar()
-	self._wait.set(0)
-
-	if geometry == 'centerscreenalways':
-	    self._centreonscreen()
-	elif geometry == 'centerscreenfirst':
-	    if self._firstShowing:
-		# Centre the window the first time it is displayed.
-		self._centreonscreen()
-	    else:
-		# Position the window at the same place it was last time.
-		self._sameposition()
-	elif geometry[:5] == 'first':
-	    if self._firstShowing:
-		self.geometry(geometry[5:])
-	    else:
-		# Position the window at the same place it was last time.
-		self._sameposition()
-	elif geometry is not None:
-	    self.geometry(geometry)
-
-	self._firstShowing = 0
-	self.deiconify()
-
         pushgrab(self._hull, globalMode, self.deactivate)
 	command = self['activatecommand']
 	if callable(command):
@@ -1051,10 +1145,6 @@ class MegaWidget(MegaArchetype):
 	# Initialise the base class (after defining the options).
 	MegaArchetype.__init__(self, parent, Tkinter.Frame)
 
-    def destroy(self):
-	del _hullToMegaWidget[self._hull]
-	self._hull.destroy()
-
 forwardmethods(MegaWidget, Tkinter.Frame, '_hull')
 
 #=============================================================================
@@ -1089,16 +1179,10 @@ def tracetk(root = None, on = 1, withStackTrace = 0, file=None):
 	tk = root.tk.getTclInterp()
     _setTkInterps(root, tk)
 
-def showbusycursor(exclude = ()):
+def showbusycursor():
 
-    # Include the Tk root window in the list of toplevels.  Can not do
-    # this earlier, because we do not know when Tkinter will be
-    # initialised.
+    _addRootToToplevelBusyInfo()
     root = Tkinter._default_root
-    if root == None:
-	root = Tkinter.Tk()
-    if not _toplevelBusyInfo.has_key(root):
-        _addToplevelBusyInfo(root)
 
     busyInfo = {
         'newBusyWindows' : [],
@@ -1107,19 +1191,20 @@ def showbusycursor(exclude = ()):
     }
     _busyStack.append(busyInfo)
 
-    # Remember the focus as it is now, before it is changed.
-    busyInfo['previousFocus'] = root.tk.call('focus')
+    if _disableKeyboardWhileBusy:
+        # Remember the focus as it is now, before it is changed.
+        busyInfo['previousFocus'] = root.tk.call('focus')
 
     if not _havebltbusy(root):
         # No busy command, so don't call busy hold on any windows.
         return
 
     for (window, winInfo) in _toplevelBusyInfo.items():
-	if (window.state() != 'withdrawn' and not winInfo['isBusy'] and
-                window not in exclude):
+	if (window.state() != 'withdrawn' and not winInfo['isBusy']
+                and not winInfo['excludeFromBusy']):
             busyInfo['newBusyWindows'].append(window)
             winInfo['isBusy'] = 1
-            _busy_hold(window)
+            _busy_hold(window, winInfo['busyCursorName'])
 
             # Make sure that no events for the busy window get
             # through to Tkinter, otherwise it will crash in
@@ -1127,22 +1212,28 @@ def showbusycursor(exclude = ()):
             # a binding on the toplevel window.
             window.tk.call('bindtags', winInfo['busyWindow'], 'Pmw_Dummy_Tag')
 
-            # Remember previous focus widget for this toplevel window
-            # and set focus to the busy window, which will ignore all
-            # events.
-            winInfo['windowFocus'] = \
-                    window.tk.call('focus', '-lastfor', window._w)
-            window.tk.call('focus', winInfo['busyWindow'])
-            busyInfo['busyFocus'] = winInfo['busyWindow']
+            if _disableKeyboardWhileBusy:
+                # Remember previous focus widget for this toplevel window
+                # and set focus to the busy window, which will ignore all
+                # keyboard events.
+                winInfo['windowFocus'] = \
+                        window.tk.call('focus', '-lastfor', window._w)
+                window.tk.call('focus', winInfo['busyWindow'])
+                busyInfo['busyFocus'] = winInfo['busyWindow']
 
     if len(busyInfo['newBusyWindows']) > 0:
-	window.update_idletasks()
+        if os.name == 'nt':
+            # NT needs an "update" before it will change the cursor.
+            window.update()
+        else:
+            window.update_idletasks()
 
 def hidebusycursor(forceFocusRestore = 0):
 
     # Remember the focus as it is now, before it is changed.
     root = Tkinter._default_root
-    currentFocus = root.tk.call('focus')
+    if _disableKeyboardWhileBusy:
+        currentFocus = root.tk.call('focus')
 
     # Pop the busy info off the stack.
     busyInfo = _busyStack[-1]
@@ -1155,40 +1246,63 @@ def hidebusycursor(forceFocusRestore = 0):
             winInfo['isBusy'] = 0
             _busy_release(window)
 
-            # Restore previous focus window for this toplevel window,
-            # but only if is still set to the busy window (it may have
-            # been changed).
-            windowFocusNow = window.tk.call('focus', '-lastfor', window._w)
-            if windowFocusNow == winInfo['busyWindow']:
-                try:
-                    window.tk.call('focus', winInfo['windowFocus'])
-                except Tkinter.TclError:
-                    # Previous focus widget has been deleted. Set focus
-                    # to toplevel window instead (can't leave focus on
-                    # busy window).
-                    window.focus_set()
+            if _disableKeyboardWhileBusy:
+                # Restore previous focus window for this toplevel window,
+                # but only if is still set to the busy window (it may have
+                # been changed).
+                windowFocusNow = window.tk.call('focus', '-lastfor', window._w)
+                if windowFocusNow == winInfo['busyWindow']:
+                    try:
+                        window.tk.call('focus', winInfo['windowFocus'])
+                    except Tkinter.TclError:
+                        # Previous focus widget has been deleted. Set focus
+                        # to toplevel window instead (can't leave focus on
+                        # busy window).
+                        window.focus_set()
 
-    # Restore the focus, depending on whether the focus had changed
-    # between the calls to showbusycursor and hidebusycursor.
-    if forceFocusRestore or busyInfo['busyFocus'] == currentFocus:
-        # The focus had not changed, so restore it to as it was before
-        # the call to showbusycursor,
-        previousFocus = busyInfo['previousFocus']
-        if previousFocus is not None:
-            try:
-                root.tk.call('focus', previousFocus)
-            except Tkinter.TclError:
-                # Previous focus widget has been deleted; forget it.
-                pass
-    else:
-        # The focus had changed, so restore it to what it had been
-        # changed to before the call to hidebusycursor.
-        root.tk.call('focus', currentFocus)
+    if _disableKeyboardWhileBusy:
+        # Restore the focus, depending on whether the focus had changed
+        # between the calls to showbusycursor and hidebusycursor.
+        if forceFocusRestore or busyInfo['busyFocus'] == currentFocus:
+            # The focus had not changed, so restore it to as it was before
+            # the call to showbusycursor,
+            previousFocus = busyInfo['previousFocus']
+            if previousFocus is not None:
+                try:
+                    root.tk.call('focus', previousFocus)
+                except Tkinter.TclError:
+                    # Previous focus widget has been deleted; forget it.
+                    pass
+        else:
+            # The focus had changed, so restore it to what it had been
+            # changed to before the call to hidebusycursor.
+            root.tk.call('focus', currentFocus)
 
 def clearbusycursor():
     while len(_busyStack) > 0:
         hidebusycursor()
 
+def setbusycursorattributes(window, **kw):
+    _addRootToToplevelBusyInfo()
+    for name, value in kw.items():
+        if name == 'exclude':
+            _toplevelBusyInfo[window]['excludeFromBusy'] = value
+        elif name == 'cursorName':
+            _toplevelBusyInfo[window]['busyCursorName'] = value
+        else:
+            raise KeyError, 'Unknown busycursor attribute "' + name + '"'
+
+def _addRootToToplevelBusyInfo():
+    # Include the Tk root window in the list of toplevels.  This must
+    # not be called before Tkinter has had a chance to be initialised by
+    # the application.
+
+    root = Tkinter._default_root
+    if root == None:
+        root = Tkinter.Tk()
+    if not _toplevelBusyInfo.has_key(root):
+        _addToplevelBusyInfo(root)
+
 def busycallback(command, updateFunction = None):
     if not callable(command):
 	raise ValueError, \
@@ -1215,13 +1329,22 @@ def displayerror(text):
 	    _errorWindow = _ErrorWindow()
 	_errorWindow.showerror(text)
 
+_root = None
+_disableKeyboardWhileBusy = 1
+
 def initialise(
         root = None,
         size = None,
         fontScheme = None,
         useTkOptionDb = 0,
         noBltBusy = 0,
+        disableKeyboardWhileBusy = None,
 ):
+    # Remember if show/hidebusycursor should ignore keyboard events.
+    global _disableKeyboardWhileBusy
+    if disableKeyboardWhileBusy is not None:
+        _disableKeyboardWhileBusy = disableKeyboardWhileBusy
+
     # Do not use blt busy command if noBltBusy is set.  Otherwise,
     # use blt busy if it is available.
     global _haveBltBusy
@@ -1241,6 +1364,24 @@ def initialise(
 	else:
 	    root = Tkinter._default_root
 
+    # If this call is initialising a different Tk interpreter than the
+    # last call, then re-initialise all global variables.  Assume the
+    # last interpreter has been destroyed - ie:  Pmw does not (yet)
+    # support multiple simultaneous interpreters.
+    global _root
+    if _root is not None and _root != root:
+        global _busyStack
+        global _errorWindow
+        global _grabStack
+        global _hullToMegaWidget
+        global _toplevelBusyInfo
+        _busyStack = []
+        _errorWindow = None
+        _grabStack = []
+        _hullToMegaWidget = {}
+        _toplevelBusyInfo = {}
+    _root = root
+
     # Trap Tkinter Toplevel constructors so that a list of Toplevels
     # can be maintained.
     Tkinter.Toplevel.title = __TkinterToplevelTitle
@@ -1249,7 +1390,7 @@ def initialise(
     # destroyed when their hull widget is destoyed and the list of
     # Toplevels can be pruned.
     Tkinter.Toplevel.destroy = __TkinterToplevelDestroy
-    Tkinter.Frame.destroy = __TkinterFrameDestroy
+    Tkinter.Widget.destroy = __TkinterWidgetDestroy
 
     # Modify Tkinter's CallWrapper class to improve the display of
     # errors which occur in callbacks.
@@ -1303,13 +1444,18 @@ class _TraceTk:
     def getTclInterp(self):
         return self.tclInterp
 
+    # Calling from python into Tk.
     def call(self, *args, **kw):
         global _callToTkReturned
         global _recursionCounter
 
         _callToTkReturned = 0
+        if len(args) == 1 and type(args[0]) == types.TupleType:
+            argStr = str(args[0])
+        else:
+            argStr = str(args)
 	_traceTkFile.write('CALL  TK> %d:%s%s' %
-                (_recursionCounter, '  ' * _recursionCounter, str(args)))
+                (_recursionCounter, '  ' * _recursionCounter, argStr))
 	_recursionCounter = _recursionCounter + 1
         try:
             result = apply(self.tclInterp.call, args, kw)
@@ -1337,6 +1483,7 @@ class _TraceTk:
             _traceTkFile.write('CALL  TK> stack:\n')
             traceback.print_stack()
 
+        _traceTkFile.flush()
         return result
 
     def __getattr__(self, key):
@@ -1355,7 +1502,9 @@ def _setTkInterps(window, tk):
 # Also keep a list of the widget which last had focus for each
 # toplevel.
 
-# Map from toplevel windows to {'isBusy', 'windowFocus', 'busyWindow'}
+# Map from toplevel windows to
+#     {'isBusy', 'windowFocus', 'busyWindow',
+#         'excludeFromBusy', 'busyCursorName'}
 _toplevelBusyInfo = {}
 
 # Pmw needs to know all toplevel windows, so that it can call blt busy
@@ -1373,7 +1522,9 @@ def _addToplevelBusyInfo(window):
     _toplevelBusyInfo[window] = {
         'isBusy' : 0,
         'windowFocus' : None,
-        'busyWindow' : busyWindow
+        'busyWindow' : busyWindow,
+        'excludeFromBusy' : 0,
+        'busyCursorName' : None,
     }
 
 def __TkinterToplevelTitle(self, *args):
@@ -1383,7 +1534,8 @@ def __TkinterToplevelTitle(self, *args):
     # about it.
     if not _toplevelBusyInfo.has_key(self):
         _addToplevelBusyInfo(self)
-	self.protocol('WM_DELETE_WINDOW', self.destroy)
+        self._Pmw_WM_DELETE_name = self.register(self.destroy, None, 0)
+	self.protocol('WM_DELETE_WINDOW', self._Pmw_WM_DELETE_name)
 
     return apply(Tkinter.Wm.title, (self,) + args)
 
@@ -1394,7 +1546,13 @@ def _havebltbusy(window):
         import PmwBlt
         _haveBltBusy = PmwBlt.havebltbusy(window)
         _busy_hold = PmwBlt.busy_hold
-        _busy_release = PmwBlt.busy_release
+        if os.name == 'nt':
+            # There is a bug in Blt 2.4i on NT where the busy window
+            # does not follow changes in the children of a window.
+            # Using forget works around the problem.
+            _busy_release = PmwBlt.busy_forget
+        else:
+            _busy_release = PmwBlt.busy_release
     return _haveBltBusy
 
 class _BusyWrapper:
@@ -1416,6 +1574,48 @@ class _BusyWrapper:
 
 #=============================================================================
 
+def drawarrow(canvas, fg, direction, tag):
+    canvas.delete(tag)
+
+    bw = (string.atoi(canvas['borderwidth']) + 
+            string.atoi(canvas['highlightthickness']))
+    width = string.atoi(canvas['width'])
+    height = string.atoi(canvas['height'])
+
+    baseOffset = 0.25
+    edgeOffset = 0.15
+
+    if direction in ('up', 'down'):
+        majorDimension = height
+        minorDimension = width
+    else:
+        majorDimension = width
+        minorDimension = height
+
+    offset = round(baseOffset * majorDimension)
+    if direction in ('down', 'right'):
+        base = bw + offset
+        apex = bw + majorDimension - offset
+    else:
+        base = bw + majorDimension - offset
+        apex = bw + offset
+
+    if minorDimension > 3 and minorDimension % 2 == 0:
+        minorDimension = minorDimension - 1
+    half = int(minorDimension * (1 - 2 * edgeOffset)) / 2
+    low = round(bw + edgeOffset * minorDimension)
+    middle = low + half
+    high = low + 2 * half
+
+    if direction in ('up', 'down'):
+        coords = (low, base, high, base, middle, apex)
+    else:
+        coords = (base, low, base, high, apex, middle)
+    kw = {'fill' : fg, 'outline' : fg, 'tag' : tag}
+    apply(canvas.create_polygon, coords, kw)
+
+#=============================================================================
+
 # Modify the Tkinter destroy methods so that it notifies us when a Tk
 # toplevel or frame is destroyed.
 
@@ -1425,24 +1625,31 @@ _hullToMegaWidget = {}
 
 def __TkinterToplevelDestroy(tkWidget):
     if _hullToMegaWidget.has_key(tkWidget):
-      mega = _hullToMegaWidget[tkWidget]
-      try:
-	  mega.destroy()
-      except:
-	  _reporterror(mega.destroy, ())
+        mega = _hullToMegaWidget[tkWidget]
+        try:
+	    mega.destroy()
+        except:
+	    _reporterror(mega.destroy, ())
     else:
-      del _toplevelBusyInfo[tkWidget]
-      Tkinter.Widget.destroy(tkWidget)
+        # Delete the busy info structure for this toplevel (if the
+        # window was created before Pmw.initialise() was called, it
+        # will not have any.
+        if _toplevelBusyInfo.has_key(tkWidget):
+            del _toplevelBusyInfo[tkWidget]
+        if hasattr(tkWidget, '_Pmw_WM_DELETE_name'):
+            tkWidget.tk.deletecommand(tkWidget._Pmw_WM_DELETE_name)
+            del tkWidget._Pmw_WM_DELETE_name
+        Tkinter.BaseWidget.destroy(tkWidget)
 
-def __TkinterFrameDestroy(tkWidget):
+def __TkinterWidgetDestroy(tkWidget):
     if _hullToMegaWidget.has_key(tkWidget):
-      mega = _hullToMegaWidget[tkWidget]
-      try:
-	  mega.destroy()
-      except:
-	  _reporterror(mega.destroy, ())
+        mega = _hullToMegaWidget[tkWidget]
+        try:
+	    mega.destroy()
+        except:
+	    _reporterror(mega.destroy, ())
     else:
-      Tkinter.Widget.destroy(tkWidget)
+        Tkinter.BaseWidget.destroy(tkWidget)
 
 #=============================================================================
 
@@ -1454,6 +1661,8 @@ class __TkinterCallWrapper:
 	self.func = func
 	self.subst = subst
 	self.widget = widget
+
+    # Calling back from Tk into python.
     def __call__(self, *args):
 	try:
 	    if self.subst:
@@ -1477,6 +1686,7 @@ class __TkinterCallWrapper:
                     argStr = str(args)
                 _traceTkFile.write('CALLBACK> %d:%s%s%s\n' %
                     (_recursionCounter, '  ' * _recursionCounter, name, argStr))
+                _traceTkFile.flush()
 	    return apply(self.func, args)
 	except SystemExit, msg:
 	    raise SystemExit, msg
@@ -1615,15 +1825,12 @@ class _ErrorWindow:
 	else:
 	    geometry = self._top.geometry()
 	    index = string.find(geometry, '+')
+            if index >= 0:
+                geom = geometry[index:]
+            else:
+                geom = None
 
-            if os.name != "nt" and index >= 0:
-		self._top.geometry(geometry[index:])
-	    self._top.deiconify()
-            if os.name == "nt":
-                # deiconify does not raise. Bug in Tk?
-                self._top.tkraise()
-                if index >= 0:
-                    self._top.geometry(geometry[index:])
+            setgeometryanddeiconify(self._top, geom)
 
 	self._updateButtons()
 
