@@ -38,12 +38,23 @@ class ComboBoxDialog(Pmw.Dialog):
 	self._combobox.pack(side='top', expand='true', fill='both',
 		padx = self['borderx'], pady = self['bordery'])
 
-	# Whenever this dialog is activated, set the focus to the
-	# ComboBox's listbox widget.
-	listbox = self.component('listbox')
-	self.configure(activatecommand = listbox.focus_set)
+        if not kw.has_key('activatecommand'):
+            # Whenever this dialog is activated, set the focus to the
+            # ComboBox's listbox widget.
+            listbox = self.component('listbox')
+            self.configure(activatecommand = listbox.focus_set)
 
 	# Check keywords and initialise options.
 	self.initialiseoptions(ComboBoxDialog)
 
+    # Need to explicitly forward this to override the stupid
+    # (grid_)size method inherited from Tkinter.Toplevel.Grid.
+    def size(self):
+	return self._combobox.size()
+
+    # Need to explicitly forward this to override the stupid
+    # (grid_)bbox method inherited from Tkinter.Toplevel.Grid.
+    def bbox(self, index):
+	return self._combobox.bbox(index)
+
 Pmw.forwardmethods(ComboBoxDialog, Pmw.ComboBox, '_combobox')
